#!/bin/sh 
if [ $# -eq 1 ] && [ $1 = "--version" ]; then
   echo pbs_version=PBSPro_10.4.5.110853
   exit 0
fi
#
#    PBS init script
#
#	Recognized arguments:
#	    start   - start PBS
#	    stop    - terminate PBS
#	    restart - terminate and start PBS
#	    status  - report PBS deamon pids
#
#
# chkconfig: 35 90 10
# description: The Portable Batch System (PBS) is a flexible workload 
# management  system. It operates on # networked, multi-platform UNIX 
# environments, including heterogeneous clusters of workstations, 
# supercomputers, and massively parallel systems. 
#
### BEGIN INIT INFO
# Provides:       pbs
# Required-Start: $network $local_fs $remote_fs $named
# Required-Stop:  $network $local_fs $remote_fs $named
# Default-Start:  3 5
# Default-Stop:   0 1 2 4 6
# Description:    Portable Batch System
### END INIT INFO
#
#  Copyright (C) 2003-2010 Altair Engineering, Inc. All rights reserved.
# 
#  ALTAIR ENGINEERING INC. Proprietary and Confidential. Contains Trade Secret
#  Information. Not for use or disclosure outside ALTAIR and its licensed
#  clients. Information contained herein shall not be decompiled, disassembled,
#  duplicated or disclosed in whole or in part for any purpose. Usage of the
#  software is only as explicitly permitted in the end user software license
#  agreement.
# 
#  Copyright notice does not imply publication.


getpid() {
	if [ -f $1 ]; then
		cat $1
	else
		echo -1
	fi
}

update_pids() {
	pbs_server_pid=`getpid ${PBS_HOME}/server_priv/server.lock`
	pbs_secondary_server_pid=`getpid ${PBS_HOME}/server_priv/server.lock.secondary`
	pbs_mom_pid=`getpid ${PBS_MOM_HOME}/mom_priv/mom.lock`
	pbs_sched_pid=`getpid ${PBS_HOME}/sched_priv/sched.lock`
	pbs_secondary_sched_pid=`getpid ${PBS_HOME}/sched_priv/sched.lock.secondary`
}

# lc_host_name - convert host name into lower case short host name

lc_host_name() 
{
  echo $1 | sed -e "s/\..*//" | sed -e "y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/"
}

# check_started - check if a particular pid is the program which is expected.
#                 pbs stores the pid of the currently running incarnation of 
#                 itself.  This function is used to see if that pid is correct 
#		  program.
# 	$1 - the pid
# 	$2 - the program name (pbs_server pbs_mom pbs_sched)
#
# return value: 0 - program is already running
#		1 - program is not running
#
: check_started
check_started() {

  ps_out=`ps -p $1 -o args 2> /dev/null | tail -1`
  if [ -z "${ps_out}" -o "`echo ${ps_out} | cut -c1`" = "[" ]  ; then
    ps_out=`ps -p $1 -o command 2> /dev/null | tail -1`
    if [ -z "${ps_out}" ]; then
	return 1
    fi
  fi

# strip out everything except executable name
  prog_name=`echo ${ps_out} | awk '{print $1}' | \
	awk -F/ '{print $NF}' | awk '{print $1}'`

  if [ "${prog_name}" = $2 ] ; then
    return 0;
  fi

  return 1;
}

# check_prog - this function checks to see if a prog is still running.  It will
#              get the pid out of the prog.lock file and run check_started 
#	       on that pid.
#
#	$1 is either "server" "mom" or "sched"
#
# return value: 0 - program is still running
#		1 - program is not running
#
: check_prog
check_prog() {

  case $1 in 
    mom) pid=${pbs_mom_pid} ;;
    server)
        if [ ${is_secondary} -eq 0 ] ; then 
            pid=${pbs_server_pid} 
        else 
            pid=${pbs_secondary_server_pid} 
        fi ;; 
    sched)
        if [ ${is_secondary} -eq 0 ] ; then
            pid=${pbs_sched_pid} ;
        else
            pid=${pbs_secondary_sched_pid} ;
        fi ;;

    *)   echo Invalid PBS daemon name: $1 >&2;
	 return 1;;
  esac

  if [ -n "${pid}" -a "${pid}" -ne -1 ] ; then
      if check_started "${pid}" "pbs_${1}" ; then
	return 0
      else
	return 1
      fi
  else
      return 1
  fi
  # Since the pid file does not exist, PBS has never been run
  return 1
}

# Look to see if restrict_user_maxsysid exists in MOM config file
# If it is there, nothing else needs to be done.
# Otherwise, see if a default can be determined from login.defs.
check_maxsys()
{
	if grep '^$restrict_user_maxsysid' \
			${PBS_MOM_HOME}/mom_priv/config > /dev/null ;then
		return
	fi

	file=/etc/login.defs
	if [ -f $file ]; then
		val=`awk '$1 == "SYSTEM_UID_MAX" {print $2}' $file`
		if [ -z "$val" ]; then
			val=`awk '$1 == "UID_MIN" {print $2 - 1}' $file`
		fi
		if [ -n "$val" ]; then
			echo '$restrict_user_maxsysid' $val \
				>> ${PBS_MOM_HOME}/mom_priv/config
		fi
	fi
}

start_pbs() {
    echo "Starting PBS"

#
# See if we need to make PBS_HOME; we do if 
# 1) PBS_HOME doesn't exist  (needmakehome=1 -> create PBS_HOME)
# 2) PBS_HOME/pbs_version doesn't exist 
# 3) if the version number in PBS_HOME/pbs_version does not match 
#    the version of the commands  (2 and 3 needmakehome=2 -> update)
#
    needmakehome=1
    if [ -d ${PBS_HOME} ] ; then
	needmakehome=2
	if [ -f ${PBS_HOME}/pbs_version ] ; then
	    if [ -x ${PBS_EXEC}/bin/qstat ] ; then
		qstatver=`${PBS_EXEC}/bin/qstat --version | sed -e "s/^.*PBSPro_//"`
		homever=`cat ${PBS_HOME}/pbs_version`
		if [ "${qstatver}" = "${homever}" ] ; then
		    needmakehome=0
		fi
	    fi
	fi
    fi

    if [ ! -x ${PBS_EXEC}/etc/pbs_habitat ] ; then
	echo "${PBS_EXEC}/etc/pbs_habitat" is missing >&2
    elif [ $needmakehome -gt 0 ] ; then
	if [ $needmakehome -eq 1 ] ; then
	    echo PBS Home directory ${PBS_HOME} does not exist.
	    if [ -x ${PBS_EXEC}/etc/pbs_habitat ] ; then
		echo Running ${PBS_EXEC}/etc/pbs_habitat to create it.
	    else
		echo ${PBS_EXEC}/etc/pbs_habitat not found, cannot create Home >&2
		exit 1
	    fi
	else
	    echo PBS Home directory ${PBS_HOME} needs updating.
	    if [ -x ${PBS_EXEC}/etc/pbs_habitat ] ; then
		echo Running ${PBS_EXEC}/etc/pbs_habitat to update it.
	    else
		echo ${PBS_EXEC}/etc/pbs_habitat not found, Home not updated
	    fi
	fi
	if ${PBS_EXEC}/etc/pbs_habitat ; then
		if [ $needmakehome -eq 1 ] ; then
			echo Home directory ${PBS_HOME} created.
		else
			echo Home directory ${PBS_HOME} updated.
		fi
	else
		exit 1
	fi
    fi

    if [ "${PBS_START_MOM}" -gt 0 ] ; then
      if check_prog "mom" ; then
	echo "PBS mom already running."
      else
        if [ -f ${PBS_EXEC}/lib/init.d/limits.pbs_mom ] ; then
            . ${PBS_EXEC}/lib/init.d/limits.pbs_mom
        fi	
	check_maxsys
	site_mom_startup
        if [ "XX${BGLDIR}" != "XX" ] && [ "XX${DB2DIR}" != "XX" ] ; then
	    env LD_LIBRARY_PATH=$DB2DIR/lib64:$DB2DIR/lib ${PBS_EXEC}/sbin/pbs_mom
	    echo "PBS Blue Gene mom"
        else
	   if  ${PBS_EXEC}/sbin/pbs_mom
           then
	      echo "PBS mom"
           else
              ret_val=$?
              echo "pbs_mom startup failed, exit ${ret_val} aborting." >&2
              exit 1
           fi
	fi
      fi
    fi

    if [ "${PBS_START_SCHED}" -gt 0 ] ; then
      if check_prog "sched" ; then
	echo "PBS scheduler already running."
      else
         if [ -f ${PBS_EXEC}/lib/init.d/limits.pbs_sched ] ; then
             . ${PBS_EXEC}/lib/init.d/limits.pbs_sched
         fi	
	 if ${PBS_EXEC}/sbin/pbs_sched
	 then
            echo "PBS sched"
         else
            ret_val=$?
            echo "pbs_sched startup failed, exit ${ret_val} aborting." >&2
            exit 1
         fi
      fi
    fi

    if [ "${PBS_START_SERVER}" -gt 0 ] ; then
      if check_prog "server" ; then
	echo "PBS Server already running."
      else
         if [ -f ${PBS_EXEC}/lib/init.d/limits.pbs_server ] ; then
             . ${PBS_EXEC}/lib/init.d/limits.pbs_server
         fi	
	 if ${PBS_EXEC}/sbin/pbs_server
	 then
            echo "PBS server"
         else
            ret_val=$?
            echo "pbs_server startup failed, exit ${retval} aborting." >&2
            exit 1
         fi
      fi
    fi

    if [ -f ${PBS_EXEC}/lib/init.d/limits.post_services ] ; then
        . ${PBS_EXEC}/lib/init.d/limits.post_services
    fi
    if [ -f /etc/redhat-release ] ; then
	touch ${redhat_subsys_filepath}
    fi
    if [ -r "${PBS_HOME}/server_priv/qmgr_startup" ]; then
      echo "pbs_server evaluating ${PBS_HOME}/server_priv/qmgr_startup"
      ${PBS_EXEC}/bin/qmgr <"${PBS_HOME}/server_priv/qmgr_startup"
    fi
}

stop_pbs() {
    echo "Stopping PBS"
    if [ "${PBS_START_SERVER}" -gt 0 ] ; then
      active_server=`lc_host_name \`${PBS_EXEC}/bin/qstat -Bf 2>/dev/null | \
	  grep "server_host = " | \
	  sed -e "s/.*server_host = //"\``
      if check_prog "server" ; then
	if [ "${host}" = "${active_server}" ]; then
	  if [ -z "${PBS_SECONDARY}" -o ${is_secondary} -eq 1 ]; then
	    echo "Shutting server down with qterm."
	  else
	    echo "This is active server, shutting down with qterm, secondary will take over."
	  fi
	  if [ -r "${PBS_HOME}/server_priv/qmgr_shutdown" ]; then
	    echo "pbs_server evaluating ${PBS_HOME}/server_priv/qmgr_shutdown"
	    ${PBS_EXEC}/bin/qmgr <"${PBS_HOME}/server_priv/qmgr_shutdown"
	  fi
	  ${PBS_EXEC}/bin/qterm -t quick
	  echo "PBS server - was pid: ${pbs_server_pid}"
	elif [ ${is_secondary} -eq 1 ]; then
	  echo "This is secondary server, killing process."
	  kill ${pbs_secondary_server_pid}
	  echo "PBS server - was pid: ${pbs_secondary_server_pid}"
	  rm -f ${PBS_HOME}/server_priv/server.lock.secondary
	else
	  echo "Killing Server."
	  kill ${pbs_server_pid}
	  echo "PBS server - was pid: ${pbs_server_pid}"
	fi
      fi
    fi
    if [ "${PBS_START_MOM}" -gt 0 ] ; then
      if check_prog "mom" ; then
	site_mom_cleanup
	kill ${pbs_mom_pid}
	echo "PBS mom - was pid: ${pbs_mom_pid}"
      fi
    fi
    if [ "${PBS_START_SCHED}" -gt 0 ] ; then
      if check_prog "sched" ; then
	if [ ${is_secondary} -eq 0 ] ; then
	  kill ${pbs_sched_pid} 
	  echo "PBS sched - was pid: ${pbs_sched_pid}"
	else
	  kill ${pbs_secondary_sched_pid}
	  echo "PBS schedx - was pid: ${pbs_secondary_sched_pid}"
	  rm -f ${PBS_HOME}/sched_priv/sched.lock.secondary
	fi
      fi
    fi
    if [ -f ${redhat_subsys_filepath} ] ; then
	  rm -f ${redhat_subsys_filepath}
    fi
}

status_pbs() {
    if [ "${PBS_START_SERVER}" -gt 0 ] ; then
      if check_prog "server" ; then
	echo "pbs_server is pid ${pbs_server_pid}"
      else
	echo "pbs server is not running."
      fi
    fi
    if [ "${PBS_START_MOM}" -gt 0 ] ; then
      if check_prog "mom" ; then
	echo "pbs_mom is pid ${pbs_mom_pid}"
      else
	echo "pbs_mom is not running"
      fi
    fi
    if [ "${PBS_START_SCHED}" -gt 0 ] ; then
      if check_prog "sched" ; then
	echo "pbs_sched is pid ${pbs_sched_pid}"
      else
	echo "pbs_sched is not running"
      fi
    fi
}

using_cpuset_mom()
{
	if type ldd > /dev/null 2>&1
	then
		ldd $PBS_EXEC/sbin/pbs_mom 2> /dev/null | grep libcpuset > /dev/null
	else
		return 1
	fi
}


# find_bgl_environment:
#
# Finds a BGLDIR path that contains Blue Gene API include
# files and library files, and DB2DIR path that has the DB2 API includes
# and libdirs.
# This function attempts to derive BGLDIR from BRIDGE_CONFIG_FILE.
#

find_bgl_environment ()
{

	bgldef=""
        if [ "XX${BRIDGE_CONFIG_FILE}" != "XX" ] ; then
        	bgldef0=`dirname $BRIDGE_CONFIG_FILE 2>/dev/null`
        	bgldef=`dirname $bgldef0 2>/dev/null`
	fi

        bglpaths="$bgldef /bgl/BlueLight/ppcfloor/bglsys"

        BGLDIR=""
        for path in $bglpaths ; do
                if [ -d  "$path" ] ; then
                        BGLDIR="$path"
                        break
                fi
        done

        db2paths="$DB2DIR /opt/IBM/db2/V8.1"
        DB2DIR=""
        for path in $db2paths ; do
                if [ -d  "$path" ] ; then
                        DB2DIR="$path"
                        break
                fi
        done

}

# called when we are starting on a SGI ICE system
# /etc/sgi-compute-node-release exists
# Make a vnodedef file with cpuset info 
sgi_ice_startup()
{
	vnodedefsdir=`dirname $vnodedefs`
	[ -d $vnodedefsdir ] || mkdir -p -m 0700 $vnodedefsdir

	sh $momscriptdir/sgiICEvnode.sh "$1"  > ${vnodedefs}.new

	if [ ! -f $vnodedefs ] || ! cmp -s $vnodedefs ${vnodedefs}.new
	then
		/bin/cp ${vnodedefs}.new $vnodedefs
	fi
	/bin/rm -f ${vnodedefs}.new
}

site_mom_startup()
{
    vnodedefs="${PBS_MOM_HOME}/mom_priv/config.d/PBSvnodedefs"
    momscriptdir="${PBS_EXEC}/lib/init.d"
    SGItopology="/proc/sgi_sn/sn_topology";
    UVSGItopology="/proc/sgi_uv/topology";
    SGIcomputenode="/etc/sgi-compute-node-release"
    
    #	If this site is using a pbs_mom that supports CPU sets ...
    if using_cpuset_mom
    then
        # and if there's an SGI topology file (Altix), 
	# generate a placement and vnode definitions file.
	if  [ -f $SGItopology -o -f $UVSGItopology ]
	then
		vnodedefsdir=`dirname $vnodedefs`
		[ -d $vnodedefsdir ] || mkdir -p -m 0700 $vnodedefsdir
		awk -f "$momscriptdir/sgigenvnodelist.awk" > $vnodedefs
	elif [ -f $SGIcomputenode ]
	then
		sgi_ice_startup cpuset
	fi
    else
	rm -f $vnodedefs
    fi
    find_bgl_environment
}

#	When shutting down PBS, if no processes are left running in
#	the /dev/cpuset/PBSPro hierarchy, tidy up by turning off the
#	cpu_exclusive and mem_exclusive flags on the sets created by
#	pbs_mom.cpuset.
PBSPro_cleanup()
{
	CPUset_root="/dev/cpuset"
	PBSPro_root="PBSPro"
	tmpfile="/tmp/cpusetlist$$"

	trap "rm -f $tmpfile" 0 1 2 3 15

	[ -d "$CPUset_root/$PBSPro_root" ] || return
	cpuset -s $PBSPro_root -r > $tmpfile
	while read set
	do
		[ -n "`cpuset -p $set`" ] && return
	done < $tmpfile

	find $CPUset_root/$PBSPro_root -type d -print |
	sed -e "s,$CPUset_root,," |
	sort -r |
	while read set
	do
		cpuset -x $set
	done
}

site_mom_cleanup()
{
	CPUset_root="/dev/cpuset"

	if [ -d $CPUset_root ] && using_cpuset_mom
	then
		PBSPro_cleanup
	fi
}

: main code
conf=${PBS_CONF_FILE:-/etc/pbs.conf}

if [ -f "${conf}" ] ; then
. "${conf}"
  if [ -z "${PBS_START_SERVER}" ] ; then
      PBS_START_SERVER=0
  fi
  if [ -z "${PBS_START_MOM}" ] ; then
      PBS_START_MOM=0
  fi
  if [ -z "${PBS_START_SCHED}" ] ; then
      PBS_START_SCHED=0
  fi
  UNIX95=1
  export UNIX95
  PBS_MOM_HOME=${PBS_MOM_HOME:-$PBS_HOME}
else
  echo "***" >&2
  echo "*** Cannot find PBS configuration file: ${conf}" >&2
  echo "***" >&2
  exit 1
fi

redhat_subsys_filepath="/var/lock/subsys/pbs"
host=`lc_host_name \`hostname\``
is_secondary=0
if [ -n "${PBS_SECONDARY}" ]; then
  sec_host=`lc_host_name ${PBS_SECONDARY}`
  if [ ${sec_host} = ${host} ]; then
    is_secondary=1
  fi
fi
update_pids

# lets see how we were called
case "`basename $0`" in
  pbs_start) start_pbs ; exit 0 ;;
  pbs_stop) stop_pbs ; exit 0 ;;
esac

case "$1" in
  start_msg) echo "Starting PBS" ;;
  stop_msg) echo "Stopping PBS" ;;
  status) status_pbs ;;
  start) start_pbs ;;
  stop) stop_pbs ;;
  restart) echo "Restarting PBS" ; stop_pbs ; 
	   echo "Waiting for shutdown to complete" ; sleep 9 ; start_pbs ;;
  *) echo "Usage: `basename $0` --version" ;
     echo "Usage: `basename $0` {start|stop|restart|status}" ; exit 1 ;;
esac
