"""
/*
 *  Copyright (C) 2003-2010 Altair Engineering, Inc. All rights reserved.
 *
 *  ALTAIR ENGINEERING INC. Proprietary and Confidential. Contains Trade Secret
 *  Information. Not for use or disclosure outside ALTAIR and its licensed
 *  clients. Information contained herein shall not be decompiled, disassembled,
 *  duplicated or disclosed in whole or in part for any purpose. Usage of the
 *  software is only as explicitly permitted in the end user software license
 *  agreement.
 *
 *  Copyright notice does not imply publication.
 *
 */
"""

__doc__ = r"""

This module captures all the python types representing the PBS Server objects
(server,queue,job,resv, etc.)
"""
from _base_types import (PbsAttributeDescriptor, PbsReadOnlyDescriptor,
                         pbs_resource, _LOG,
                         )
import _pbs_v1
from _pbs_v1 import (_event_accept, _event_reject,
                    _event_param_mod_allow, _event_param_mod_disallow,
		    iter_nextfunc)

from _exc_types import *
#:------------------------------------------------------------------------
#                       JOB TYPE
#:-------------------------------------------------------------------------
class _job(object):
    """
    This represents a PBS job.
    """

    attributes = PbsReadOnlyDescriptor('attributes', {})
    _attributes_hook_set = {}

    def __new__(cls,value):
	# the way to clear the class variable
	_job._attributes_hook_set = {}
	return object.__new__(cls, value)
    
    def __init__(self,id):
        """__init__"""

        self.id = id
	self._readonly = False
    #: m(__init__)

    def __str__(self):
        """String representation of the object"""

        return str(self.id)
    #: m(__str__)

    def __setattr__(self, name, value):
	if name == "_readonly":
	    if _pbs_v1.in_python_mode() and \
				hasattr(self, "_readonly") and not value:
	      raise BadAttributeValueError("_readonly can only be set to True!")
	elif not _job.attributes.has_key(name):
	    raise UnsetAttributeNameError("job attribute '%s' not found" % (name,))
        super(_job,self).__setattr__(name, value)

	# attributes that are set in python mode will be reflected in
	# _attributes_hook_set dictionary.
	# For example,
	# _attributes_hook_set[<job object>]=['Priority', 'comment']
	# if 'comment' or 'Priority' has been assigned a value within the hook
	# script, or been unset.

	if _pbs_v1.in_python_mode():
	    if not self._attributes_hook_set.has_key(self):
	        self._attributes_hook_set[self] = {}
	    # using a dictionary value as easier to search for keys
	    self._attributes_hook_set[self].update({name : None})
	
    #: m(__setattr__)	
    

_job.id = PbsAttributeDescriptor(_job, 'id', "",(str,))
#: C(job)


#:-------------------------------------------------------------------------
#                       RESERVATION TYPE
#:-------------------------------------------------------------------------
class _resv(object):
    """
    This represents a PBS reservation entity.
    """
    
    attributes = PbsReadOnlyDescriptor('attributes', {})
    _attributes_hook_set = {}
    attributes_readonly = PbsReadOnlyDescriptor('attributes_readonly',
			[])
    
    def __new__(cls,value):
	# the way to clear the class variable
	_resv._attributes_hook_set = {}
	return object.__new__(cls, value)

    def __init__(self,resvid):
        """__init__"""

        self.resvid = resvid
	self._readonly = False
    #: m(__init__)

    def __str__(self):
        """String representation of the object"""

        return str(self.resvid)
    #: m(__str__)

    def __setattr__(self, name, value):
	if (name == "_readonly"):
	    if _pbs_v1.in_python_mode() and \
	    			hasattr(self, "_readonly") and not value:
	      raise BadAttributeValueError("_readonly can only be set to True!")
	elif not _resv.attributes.has_key(name):
	    raise UnsetAttributeNameError("resv attribute '%s' not found" % (name,))
	elif name in _resv.attributes_readonly and \
				_pbs_v1.in_python_mode() and \
						_pbs_v1.in_site_hook():
	    # readonly under a SITE hook
	    raise BadAttributeValueError("resv attribute '%s' is readonly" % (name,))
        super(_resv,self).__setattr__(name, value)

	# attributes that are set in python mode will be reflected in
	# _attributes_hook_set dictionary.
	# For example,
	# _attributes_hook_set[<resv object>]=['reserve_start', 'reserve_end']
	# if 'reserve_start' or 'reserve_end' has been assigned a value within
	# the hook script, or been unset.

	if _pbs_v1.in_python_mode():
	    if not self._attributes_hook_set.has_key(self):
	        self._attributes_hook_set[self] = {}
	    # using a dictionary value as easier to search for keys
	    self._attributes_hook_set[self].update({name : None})
    #: m(__setattr__)
    
#: C(resv)
_resv.resvid = PbsAttributeDescriptor(_resv, 'resvid', "", (str,))

#: End (resv) setting class attributes

#:-------------------------------------------------------------------------
#                       QUEUE TYPE
#:-------------------------------------------------------------------------
class _queue(object):
    """
    This represents a PBS queue.
    """
    
    attributes = PbsReadOnlyDescriptor('attributes', {})
    #name = PbsAttributeDescriptor(queue, 'name', "", (str,))
    def __init__(self,name):
        """__init__"""
        #: ok, descriptor is set.
        self.name = name
	self._readonly = False
    #: m(__init__)

    def __str__(self):
        """String representation of the object"""

        return str(self.name)
    #: m(__str__)

    def __setattr__(self, name, value):
	if (name == "_readonly"):
	    if _pbs_v1.in_python_mode() and \
	    		hasattr(self, "_readonly") and not value:
	      raise BadAttributeValueError("_readonly can only be set to True!")
	elif not _queue.attributes.has_key(name):
	    raise UnsetAttributeNameError("queue attribute '%s' not found" % (name,))
        super(_queue,self).__setattr__(name, value)
    #: m(__setattr__)

    def job(self, jobid):
        """Return a job object representing jobid that belongs to queue"""

        return _pbs_v1.get_job(jobid, self.name)
    #: m(job)

    def jobs(self):
        """
    	Returns an iterator that loops over the list of jobs on this queue.
        """

        return pbs_iter("jobs", "",  self.name)
    #: m(jobs)
    
#: C(_queue)

_queue.name = PbsAttributeDescriptor(_queue, 'name', "", (str,))

#: End (queue) setting class attributes

#:-------------------------------------------------------------------------
#                       Server TYPE
#:-------------------------------------------------------------------------
class _server(object):
    """
    This represents the PBS server entity.
    """
    
    attributes = PbsReadOnlyDescriptor('attributes', {})
    
    def __init__(self,name):
        """__init__"""

        self.name = name
	self._readonly = False
    #: m(__init__)

    def __str__(self):
        """String representation of the object"""

        return str(self.name)
    #: m(__str__)

    def queue(self, qname):
        """
	queue(strQname)
	    strQname -  name of a PBS queue (without the @host part) to query.

          Returns a queue object representing the queue <queue name> that is
	  managed by server s.
        """
	if qname.find("@") != -1:
            raise AssertionError(\
	       "Got '%s', please specify a queue name only (no @)" % (qname,))
        return _pbs_v1.get_queue(qname)
    #: m(queue)

    def job(self, jobid):
        """
        job(strJobid)
	    strJobid - PBS jobid to query.
          Returns a job object representing jobid
        """
        return _pbs_v1.get_job(jobid)
    #: m(job)

    def resv(self, resvid):
        """Return a resv object representing resvid"""
		 
	return _pbs_v1.get_resv(resvid)
    #: m(resv)

    def jobs(self):
        """
    	Returns an iterator that loops over the list of jobs on this server.
        """

        return pbs_iter("jobs", "",  "")
    #: m(jobs)

    def queues(self):
        """
        Returns an iterator that loops over the list of queues on this server.
        """

        return pbs_iter("queues", "",  "")
    #: m(queues)

    def resvs(self):
        """
        Returns an iterator that loops over the list of reservations on this
	server.
        """
	return pbs_iter( "resvs", "", "")
    #: m(resvs)

    def __setattr__(self, name, value):
	if (name == "_readonly"):
	    if _pbs_v1.in_python_mode() and \
	    		hasattr(self, "_readonly") and not value:
	      raise BadAttributeValueError("_readonly can only be set to True!")
	elif not _server.attributes.has_key(name):
	    raise UnsetAttributeNameError("server attribute '%s' not found" % (name,))
        super(_server,self).__setattr__(name, value)
    #: m(__setattr__)

    
#: C(server)
_server.name = PbsAttributeDescriptor(_server, 'name', "", (str,))
#: End (server) setting class attributes

#:-------------------------------------------------------------------------
#                       Event TYPE
#:-------------------------------------------------------------------------
class _event(object):
    """
    This represents the event that the current hook is responding to.
    """
    #: the below is used for attribute type acess
    attributes     = PbsReadOnlyDescriptor('attributes', {})
    
    def __init__(self, type, rq_user, rq_host):
        """__init__"""
        self.type = type
	self.requestor = rq_user
	self.requestor_host = rq_host
	self._readonly = False
    #: m(__init__)

    def accept(self, ecode=0):
	"""
	accept([ecode])
	   Terminates hook execution and causes PBS to perform the
	   associated event request action. If [ecode] argument is given,
	   it will be used as the value for the SystemExit exception, else
	   a value of 0 is used.

	   This terminates hook execution by throwing a SystemExit exception.
	"""
	_event_accept()
	_event_param_mod_disallow()
	raise SystemExit, str(ecode)
    #: m(__accept__)

    def reject(self, emsg="", ecode=255):
	"""
	reject([msg])
	   Terminates hook execution and instructs PBS to not perform the
	   associated event request action. If [msg] argument is given, it
     	   will be shown in the appropriate PBS daemon log, and the STDERR
	   of the PBS command that caused this event to take place.
	   If [ecode] argument is given, if will be used as the value for 
	   the SystemExit exception, else a value of 255 is used.
	
	   This terminates hook execution by throwing a SystemExit exception.
	"""
	_event_reject(emsg)
	_event_param_mod_disallow()
	raise SystemExit, str(ecode)
    #: m(__reject__)

    def __getattr__(self, key):
        if self._param.__contains__(key):
            return self._param[key]
        # did not find <key>
	raise EventIncompatibleError
    #: m(__getattr__)

    def __setattr__(self, name, value):
	if (name == "_readonly"):
	    if _pbs_v1.in_python_mode() and \
	    		hasattr(self, "_readonly") and not value:
	      raise BadAttributeValueError("_readonly can only be set to True!")
	elif _pbs_v1.in_python_mode() and self._param.__contains__(name):
	    raise BadAttributeValueError("event attribute '%s' is readonly" % (name,))
	elif not _event.attributes.has_key(name):
	    raise UnsetAttributeNameError("event attribute '%s' not found" % (name,))
	super(_event,self).__setattr__(name, value)
    #: m(__setattr__)
    
#: C(event)
_event.type = PbsAttributeDescriptor(_event, 'type', None, (int,))
_event.hook_name = PbsAttributeDescriptor(_event, 'hook_name', "", (str,))
_event.hook_type = PbsAttributeDescriptor(_event, 'hook_type', "", (str,))
_event.requestor = PbsAttributeDescriptor(_event, 'requestor', "", (str,))
_event.requestor_host = PbsAttributeDescriptor(_event, 'requestor_host', "", (str,))
_event._param = PbsAttributeDescriptor(_event, '_param', {}, (dict,))
#: End (event) setting class attributes

#:-------------------------------------------------------------------------
#                       PBS Iterator Type
#:-------------------------------------------------------------------------

class pbs_iter(object):
    """
    This represents an iterator for looping over a list of PBS objects.
    Pbs_obj_name can be: queues, jobs, resvs
    Pbs_filter1 is usually the <server_name> where the queues,
 		jobs, or resvs reside. A <server_name> of ""
		means the local server host.
    Pbs_filter2 can be any string that can further restrict the list
		being referenced. For example, this can be set to
		some <queue_name>, to have the iterator represents
		a list of jobs on <queue_name>@<server_name>
    """
    def __init__(self, pbs_obj_name, pbs_filter1, pbs_filter2):
        self.obj_name = pbs_obj_name
        self.filter1 = pbs_filter1
        self.filter2 = pbs_filter2
        # argument 1 below tells C function were inside __init__
        _pbs_v1.iter_nextfunc(self, 1, pbs_obj_name, pbs_filter1, pbs_filter2) 

    def __iter__(self):
        return self

    def next(self):
        # argument 0 below tells C function were inside next
        return _pbs_v1.iter_nextfunc(self, 0, self.obj_name, self.filter1,
								self.filter2)
#: C(pbs_iter)
