#         Portable Batch System (PBS) Software License
#
#
#  Copyright (C) 2003-2010 Altair Engineering, Inc. All rights reserved.
# 
#  ALTAIR ENGINEERING INC. Proprietary and Confidential. Contains Trade Secret
#  Information. Not for use or disclosure outside ALTAIR and its licensed
#  clients. Information contained herein shall not be decompiled, disassembled,
#  duplicated or disclosed in whole or in part for any purpose. Usage of the
#  software is only as explicitly permitted in the end user software license
#  agreement.
#
#  Copyright notice does not imply publication.
#
#
################################################################################
# This file contains a set of procedures for defining and manipulating 
# X resources for xpbs. This is a piece code taken from Brent B. Welch's,
# "Practical Programming in Tcl and Tk", Prentice Hall, 1995, pp. 329-338.
proc Pref_Init { userDefaults appDefaults } {
  global pref

  set pref(uid) 0; # for a unique identifier for widgets
  set pref(userDefaults) $userDefaults
  set pref(appDefaults) $appDefaults

  PrefReadFile $appDefaults startup
  if [file exists $userDefaults] {
    PrefReadFile $userDefaults user
  }
}
proc PrefReadFile {basename level} {
  if [catch {option readfile $basename $level} err] {
    InfoBox_sendmsg "Error in $basename: $err" 1
  }
 
  if { [catch {tk colormodel .}] == 0 } {
    if { [string compare [tk colormodel .] "color"] == 0 } {
      if [file exists $basename-color] {
        if [catch {option $basename-color $level} err] {
	   InfoBox_sendmsg "Error in $basename-color: $err" 1
        }	
      }
    } else {
      if [file exists $basename-mono] {
        if [catch {option readfile $basename-mono $level} err] {
           InfoBox_sendmsg "Error in $basename-mono: $err" 1
        }
      }
    }
  }
}
proc PrefVar { item } { lindex $item 0 }
proc PrefXres { item } { lindex $item 1 }
proc PrefDefault { item } { lindex $item 2 }
proc PrefComment { item } { lindex $item 3 }
proc PrefHelp { item } { lindex $item 4 }

proc Pref_Add { prefs } {

  global pref init
  append pref(items) $prefs
  foreach item $prefs {
    set varName [PrefVar $item]
    set xresName [PrefXres $item]
    set value [PrefValue $varName $xresName]

    if {$value == {}} {
      set default [PrefDefault $item]
      if {[llength $default] > 1} {
	if { [string compare [lindex $default 0] "CHOICE"] == 0 } {
          PrefValueSet $varName [lindex $default 1]
        } else {
	  PrefValueSet $varName $default
        }
      } else {
	if {[string compare $default "OFF"] == 0} {
	  PrefValueSet $varName 0
	} elseif {[string compare $default "ON"] == 0} {
	  PrefValueSet $varName 1
        } else {
	  PrefValueSet $varName $default
        }
      }
    }
    set init($xresName) [PrefValue $varName $xresName]
  }
}

# PrefValue returns the X resource database value.
proc PrefValue { varName xres } {
  upvar #0 $varName var
  if [info exists var] {
    return $var
  }
  set var [option get . $xres {}]
}
# PrefValueSet defines a variable in the global scope.
proc PrefValueSet { varName value } {
  upvar #0 $varName var
  set var $value
}
proc PrefSave {} {
  global pref preferences

  if [catch { 
	set old [open $pref(userDefaults) r]
	set oldValues [split [read $old] \n]
	close $old
  }] {
    set oldValues {}
  }

  if [catch {open $pref(userDefaults).new w} out] {
    InfoBox_sendmsg "Cannot save in $pref(userDefaults).new: $out" 1
    return 
  }
  foreach line $oldValues {
    if {[string compare $line \
	  "!!! Lines below here are automatically added"] == 0} {
	break
    } else {
        puts $out $line
    }
  }
  puts $out "!!! Lines below here are automatically added"
  puts $out "!!! [exec date]"
  foreach item $preferences {
    set varName [PrefVar $item]
    set xresName [PrefXres $item]
    set value [PrefValue $varName $xresName]
    puts $out [format "%s\t%s" *${xresName}: $value]
  }
  close $out
  set new [glob $pref(userDefaults).new]
  set old [file root $new]
  if [catch {exec mv -f $new $old} err] {
    InfoBox_sendmsg "Cannot install $new: $err"
  }
}
# prefDoIt: returns 1 if a change in the list of preference parameters is
#           detected and thus, necessitating saving current settings in
#           some preferences file.   
proc prefDoIt {} {
  global init preferences

  foreach pref $preferences {
    set varName [PrefVar $pref]
    set resName [PrefXres $pref]
    upvar #0 $varName var
    if {[string compare $resName "sitesInfo"] == 0} {
	  if {[listcomp $var $init($resName)] != 0} {
      		flush stdout
      		return 1
	  }
    } else {
	  if {[string compare $var $init($resName)] != 0} {
      		flush stdout
      		return 1
	  }	
    }
  }
  return 0
}

# default_server: returns the correct default server name using the following
#		algorithm:
#		if 'given' server name is the same as 'default_server_name'
#		but does not match the one specified in 'default_conf_file'
#		or 'default_server_file' (in that order),
#		then the latter is returned.
proc default_server { given } {
	global default_server_name default_server_file default_conf_file

	set actual $given

	set did_conf 0

	if { ([string compare $given "PBS_DEFAULT_SERVER"] == 0) || \
		([string length $default_server_file] > 0 && \
	     	 [string length $default_conf_file] > 0 && \
	     	 [string length $default_server_name] > 0 && \
	     	 [string compare $default_server_name $given] == 0) } {

		if { ![catch {open $default_conf_file r} f] } {
			while { [gets $f line] > 0 } {
				set line_split [split $line "="]
				set var [lindex $line_split 0]
				set val [lindex $line_split 1]

		     		if { [string compare $var "PBS_SERVER"] == 0 } {
				     if { [string compare $val $given] != 0 } {
					set actual $val
					set did_conf 1
				     }
				     break
			        }
			}
			close $f
		}

		if { !$did_conf } {
			if { ![catch {open $default_server_file r} f] } {
				if { [gets $f line] > 0 && \
		     			[string compare $line $given] != 0 } {
					set actual $line
				}
				close $f
			}
		}
	}

	return $actual
}

# default_node: returns the correct default node given the actual 'server'
#		using the algorithm:
#		if specified 'server' name does not match 'default_server_name',
#		    but 'given' node name matches 'default_server_name' name,
#		then
#			return the 'server' name itself.
proc default_node { server given } {
	global default_server_name

	set actual $given

	if { ([string compare $given "PBS_DEFAULT_SERVER"] == 0) || \
	     ([string length $default_server_name] > 0 && \
	      [string compare $default_server_name $server] != 0) } {

		# means 'server' name has been changed
		if { [string compare $given $default_server_name] == 0 } {
			set actual $server
		}
	}

	return $actual
}

# prefLoadSitesInfo: any badly formed sites info is automatically ignored.
proc prefLoadSitesInfo {} {
	global sitesinfo sysview sysnodes queryTable sysinfo

	foreach s $sitesinfo {
		set s_list [split $s $sysinfo(rcSiteInfoDelimeterChar)]

		if {[llength $s_list] != 7} {
			continue	;# badly-formed lines   
		}
		set sitename [lindex $s_list 0]

		set sitevtype [lindex $s_list 1]
		if {[string compare $sitevtype "ICON"] != 0 && \
		    	[string compare $sitevtype "FULL"] != 0} {
			continue
		}
		set servername [default_server [lindex $s_list 2]]
		set serverlabel [default_node $servername [lindex $s_list 3]]
		set nodename [default_node $servername [lindex $s_list 4]]

		if {[string compare $nodename ""] == 0} {
			set sysview($sitename) $sitevtype
			set sysnodes($sitename,$servername) $serverlabel
			continue
		}

		set nodetype [lindex $s_list 5]
		if {[string compare $nodetype "MOM"] != 0 && \
		  	[string compare $nodetype "NOMOM"] != 0} {
			continue
		}
		set querylist [lindex $s_list 6]

		set sysview($sitename) $sitevtype
		if {![info exists sysnodes($sitename,$servername)]} {
			lappend sysnodes($sitename,$servername) \
					     $serverlabel "$nodename $nodetype"
		} else {
			set newval [lreplace $sysnodes($sitename,$servername) \
							       0 0 $serverlabel]
			set foundMatch 0
			foreach v [lrange $newval 1 end] {
				set ninfo [join $v " "]
				set newninfo "$nodename $nodetype"
				if {[string compare $ninfo $newninfo] == 0} {
					set foundMatch 1
					break
				}
			}
			if {!$foundMatch} {
				lappend newval "$nodename $nodetype"
			}
			set sysnodes($sitename,$servername) $newval
		}
		foreach qi $querylist {
			set queryexpr [list [lindex $qi 0]]
			set querylabel [list [lindex $qi 1]]
			set querytype [list [lindex $qi 2]]

			if {[string compare $querytype "SCALE"] != 0 && \
			    [string compare $querytype "TEXT"] != 0 } {
				continue
			}

			if {![info exists queryTable($sitename,$nodename)]} { 
				lappend queryTable($sitename,$nodename) \
			 	      	     "$queryexpr $querylabel $querytype"
			} else {
				set foundMatch 0
				foreach q  $queryTable($sitename,$nodename) {
					set qexpr [lindex $q 0]
					set qtype [lindex $q 2]
					set qinfo "$qexpr $qtype"
					set newqinfo "$queryexpr $querytype"
					if {[string compare $qinfo $newqinfo] \
									 == 0} {
						set foundMatch 1
						break
					}
				}
			  	if {!$foundMatch} {
				      lappend  queryTable($sitename,$nodename) \
			 	             "$queryexpr $querylabel $querytype"
			  	}
			}
		}
	}
}
proc prefSaveSitesInfo {} {
	global sysview sysnodes queryTable sitesinfo sysinfo

	set SEP $sysinfo(rcSiteInfoDelimeterChar)

	set i 0
	foreach s [array names sysview] {

		foreach srv [array names sysnodes] {
			set srv_list [split $srv ","]
			set sitename [lindex $srv_list 0]
			set srvname  [lindex $srv_list 1]
			set srvlabel [lindex $sysnodes($srv) 0]
			set srvnodes [lrange $sysnodes($srv) 1 end]
			if {[string compare $s $sitename] == 0} {
				foreach ninfo $srvnodes {
					set nname [lindex $ninfo 0]
					set ntype [lindex $ninfo 1]
					if [info exists queryTable($s,$nname)] {
						set line($i) "$s$SEP$sysview($s)$SEP$srvname$SEP$srvlabel$SEP$nname$SEP$ntype$SEP$queryTable($s,$nname)"
					} else {
						set line($i) "$s$SEP$sysview($s)$SEP$srvname$SEP$srvlabel$SEP$nname$SEP$ntype$SEP"
					}
					incr i
				}
				if {[llength $srvnodes] == 0} {
					set line($i) "$s$SEP$sysview($s)$SEP$srvname$SEP$srvlabel$SEP$SEP$SEP"
					incr i
				}
			}
		}
	}

	set sitesinfo ""
	for {set i 0} {$i < [array size line]} {incr i} {
		lappend sitesinfo $line($i)
	}
}
